@@ -1,25 +1,20 @@
+import argparse
+import io
 import logging
-
 import os
 import sys
-import argparse
-
-try:
-    # Needed for Python < 3.3, works up to 3.8
-    import xml.etree.cElementTree as etree
-except ImportError:
-    # Python 3.9 onwards
-    import xml.etree.ElementTree as etree
+import xml.etree.ElementTree as etree
 
 from diff_cover import DESCRIPTION, VERSION
+from diff_cover.config_parser import Tool, get_config
 from diff_cover.diff_reporter import GitDiffReporter
 from diff_cover.git_diff import GitDiffTool
 from diff_cover.git_path import GitPathTool
 from diff_cover.report_generator import (
     HtmlReportGenerator,
-    StringReportGenerator,
     JsonReportGenerator,
     MarkdownReportGenerator,
+    StringReportGenerator,
 )
 from diff_cover.violationsreporters.violations_reporter import XmlCoverageReporter
 
@@ -40,6 +35,10 @@ COVERAGE_XML_HELP = "XML coverage report"
 DIFF_RANGE_NOTATION_HELP = (
     "Git diff range notation to use when comparing branches, defaults to '...'"
 )
+QUIET_HELP = "Only print errors and failures"
+SHOW_UNCOVERED = "Show uncovered lines on the console"
+INCLUDE_UNTRACKED_HELP = "Include untracked files"
+CONFIG_FILE_HELP = "The configuration file to use"
 
 LOGGER = logging.getLogger(__name__)
 
@@ -64,37 +63,35 @@ def parse_coverage_args(argv):
 
     parser.add_argument("coverage_xml", type=str, help=COVERAGE_XML_HELP, nargs="+")
 
-    output_format = parser.add_mutually_exclusive_group()
-
-    output_format.add_argument(
+    parser.add_argument(
         "--html-report",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=HTML_REPORT_HELP,
     )
 
-    output_format.add_argument(
+    parser.add_argument(
         "--json-report",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=JSON_REPORT_HELP,
     )
 
-    output_format.add_argument(
+    parser.add_argument(
         "--markdown-report",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=MARKDOWN_REPORT_HELP,
     )
 
+    parser.add_argument(
+        "--show-uncovered", action="store_true", default=None, help=SHOW_UNCOVERED
+    )
+
     parser.add_argument(
         "--external-css-file",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=CSS_FILE_HELP,
     )
 
@@ -102,7 +99,6 @@ def parse_coverage_args(argv):
         "--compare-branch",
         metavar="BRANCH",
         type=str,
-        default="origin/master",
         help=COMPARE_BRANCH_HELP,
     )
 
@@ -111,16 +107,23 @@ def parse_coverage_args(argv):
     )
 
     parser.add_argument(
-        "--ignore-staged", action="store_true", default=False, help=IGNORE_STAGED_HELP
+        "--ignore-staged", action="store_true", default=None, help=IGNORE_STAGED_HELP
     )
 
     parser.add_argument(
         "--ignore-unstaged",
         action="store_true",
-        default=False,
+        default=None,
         help=IGNORE_UNSTAGED_HELP,
     )
 
+    parser.add_argument(
+        "--include-untracked",
+        action="store_true",
+        default=None,
+        help=INCLUDE_UNTRACKED_HELP,
+    )
+
     parser.add_argument(
         "--exclude", metavar="EXCLUDE", type=str, nargs="+", help=EXCLUDE_HELP
     )
@@ -130,7 +133,6 @@ def parse_coverage_args(argv):
         metavar="DIRECTORY",
         type=str,
         nargs="+",
-        default=["src/main/java", "src/test/java"],
         help=SRC_ROOTS_HELP,
     )
 
@@ -138,23 +140,41 @@ def parse_coverage_args(argv):
         "--diff-range-notation",
         metavar="RANGE_NOTATION",
         type=str,
-        default="...",
         choices=["...", ".."],
         help=DIFF_RANGE_NOTATION_HELP,
     )
 
-    parser.add_argument(
-        "--version", action="version", version="diff-cover {}".format(VERSION)
-    )
+    parser.add_argument("--version", action="version", version=f"diff-cover {VERSION}")
 
     parser.add_argument(
         "--ignore-whitespace",
         action="store_true",
-        default=False,
+        default=None,
         help=IGNORE_WHITESPACE,
     )
 
-    return vars(parser.parse_args(argv))
+    parser.add_argument(
+        "-q", "--quiet", action="store_true", default=None, help=QUIET_HELP
+    )
+
+    parser.add_argument(
+        "-c", "--config-file", help=CONFIG_FILE_HELP, metavar="CONFIG_FILE"
+    )
+
+    defaults = {
+        "show_uncovered": False,
+        "compare_branch": "origin/main",
+        "fail_under": "0",
+        "ignore_staged": False,
+        "ignore_unstaged": False,
+        "ignore_untracked": False,
+        "src_roots": ["src/main/java", "src/test/java"],
+        "ignore_whitespace": False,
+        "diff_range_notation": "...",
+        "quiet": False,
+    }
+
+    return get_config(parser=parser, argv=argv, defaults=defaults, tool=Tool.DIFF_COVER)
 
 
 def generate_coverage_report(
@@ -166,10 +186,13 @@ def generate_coverage_report(
     markdown_report=None,
     ignore_staged=False,
     ignore_unstaged=False,
+    include_untracked=False,
     exclude=None,
     src_roots=None,
     diff_range_notation=None,
     ignore_whitespace=False,
+    quiet=False,
+    show_uncovered=False,
 ):
     """
     Generate the diff coverage report, using kwargs from `parse_args()`.
@@ -179,6 +202,7 @@ def generate_coverage_report(
         git_diff=GitDiffTool(diff_range_notation, ignore_whitespace),
         ignore_staged=ignore_staged,
         ignore_unstaged=ignore_unstaged,
+        include_untracked=include_untracked,
         exclude=exclude,
     )
 
@@ -197,18 +221,19 @@ def generate_coverage_report(
             with open(css_file, "wb") as output_file:
                 reporter.generate_css(output_file)
 
-    elif json_report is not None:
+    if json_report is not None:
         reporter = JsonReportGenerator(coverage, diff)
         with open(json_report, "wb") as output_file:
             reporter.generate_report(output_file)
 
-    elif markdown_report is not None:
+    if markdown_report is not None:
         reporter = MarkdownReportGenerator(coverage, diff)
         with open(markdown_report, "wb") as output_file:
             reporter.generate_report(output_file)
 
-    reporter = StringReportGenerator(coverage, diff)
-    output_file = sys.stdout.buffer
+    # Generate the report for stdout
+    reporter = StringReportGenerator(coverage, diff, show_uncovered)
+    output_file = io.BytesIO() if quiet else sys.stdout.buffer
 
     # Generate the report
     reporter.generate_report(output_file)
@@ -217,16 +242,19 @@ def generate_coverage_report(
 
 def main(argv=None, directory=None):
     """
-    Main entry point for the tool, used by setup.py
+    Main entry point for the tool, script installed via pyproject.toml
     Returns a value that can be passed into exit() specifying
     the exit code.
     1 is an error
     0 is successful run
     """
-    logging.basicConfig(format="%(message)s")
-
     argv = argv or sys.argv
     arg_dict = parse_coverage_args(argv[1:])
+
+    quiet = arg_dict["quiet"]
+    level = logging.ERROR if quiet else logging.WARNING
+    logging.basicConfig(format="%(message)s", level=level)
+
     GitPathTool.set_cwd(directory)
     fail_under = arg_dict.get("fail_under")
     percent_covered = generate_coverage_report(
@@ -238,17 +266,19 @@ def main(argv=None, directory=None):
         css_file=arg_dict["external_css_file"],
         ignore_staged=arg_dict["ignore_staged"],
         ignore_unstaged=arg_dict["ignore_unstaged"],
+        include_untracked=arg_dict["include_untracked"],
         exclude=arg_dict["exclude"],
         src_roots=arg_dict["src_roots"],
         diff_range_notation=arg_dict["diff_range_notation"],
         ignore_whitespace=arg_dict["ignore_whitespace"],
+        quiet=quiet,
+        show_uncovered=arg_dict["show_uncovered"],
     )
 
     if percent_covered >= fail_under:
         return 0
-    else:
-        LOGGER.error("Failure. Coverage is below {}%.".format(fail_under))
-        return 1
+    LOGGER.error("Failure. Coverage is below %i%%.", fail_under)
+    return 1
 
 
 if __name__ == "__main__":
